@@ -12,7 +12,6 @@ of the License, or (at your option) any later version.
 [cpscotti, roboscotti, scotti@ieee.org]
 
 */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -32,6 +31,9 @@ of the License, or (at your option) any later version.
 
 #define DESKTOP_CMD_EXEC_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE (obj, DESKTOP_CMD_EXEC_TYPE, DesktopCmdExecPrivate))
 
+#define DESKTOP_CMD_EXEC_SETTINGS_FILE "/.desktop_cmd_exec"
+#define DESKTOP_CMD_EXEC_SETTINGS_VERSION 0.6
+
 //main settings window
 #define NON_GTK_RESPONSE_ADD_CMD 10
 #define NON_GTK_RESPONSE_EDIT_CMD 20
@@ -49,11 +51,11 @@ of the License, or (at your option) any later version.
 #define SIZE_WIDTH_ALL 800
 #define SIZE_HEIGHT_LINE 40
 
-void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean reset);
+void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean newAll, gboolean newInstance);
 gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self);
 
 //Dialogs prototypes:
-void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new);
+void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new, gint curr);
 void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self);
 
 //Auxiliary functions' prototypes:
@@ -71,145 +73,200 @@ struct _DesktopCmdExecPrivate
 	
 	GtkWidget *cmdTitle_lb;
 	GtkWidget *cmdResult_lb;
-	
+
 	gboolean isPressed;
 
-	/* new */
-	gchar ** c_titles;
-	gchar ** c_commands;
-	guint c_size;
-	guint c_current;
-	
-	gboolean updOnClick;
-	gboolean updOnDesktop;
+	/* widget's instance identification str */
+	gchar * widgetID;
 
-	guint updNeworkPolicy;
+	//config data
 
-	gdouble widthRatio;
+	//global data
+		gchar ** c_titles;
+		gchar ** c_commands;
+		guint c_size;
 
-	//Update on timer data
-	guint delaySeconds;
-	guint delayIndex;//index of delaySeconds in the array defined inside "desktop_cmd_exec_settings"
+	//instance data
+		gboolean updOnClick;
+		gboolean updOnDesktop;
+		guint updNeworkPolicy;
+		gdouble widthRatio;
+		
+		gchar * instanceTitle;
+		gchar * instanceCmd;
 
-	//timer process id; used to stop when closing app or changing delay
-	guint updateTimerID;
+		//Update on timer data
+		guint delaySeconds;
+		guint delayIndex;//index of delaySeconds in the array defined inside "desktop_cmd_exec_settings"
 
-	//network connection query ptr
-	ConIcConnection *connection;
-	gboolean isConnected;
+		//timer process id; used to stop when closing app or changing delay
+		guint updateTimerID;
+
+		//network connection query ptr
+		ConIcConnection *connection;
+		gboolean isConnected;
 };
 
 HD_DEFINE_PLUGIN_MODULE (DesktopCmdExec, desktop_cmd_exec, HD_TYPE_HOME_PLUGIN_ITEM);
 
 
 //Settings file functions
+
 void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 {
-// 	g_warning("desktop_cmd_exec_read_settings ");
 	gchar *filename;
 	gboolean fileExists;
 	GKeyFile *keyFile;
 
-	gboolean gotAnErrorFlag;
-	gotAnErrorFlag = FALSE;
+	gboolean noSettingsFlag;
+	gboolean noInstanceKeysFlag;
+	
+	noSettingsFlag = FALSE;
+	noInstanceKeysFlag = FALSE;
+
+	if(self->priv->widgetID == NULL)
+	{
+		g_warning("Widget instance not initialized, not reading settings..");
+		return;
+	}
+	
 
 	keyFile = g_key_file_new();
-	filename = g_strconcat (HOME_DIR, "/.desktop_cmd_exec", NULL);
+	filename = g_strconcat (HOME_DIR, DESKTOP_CMD_EXEC_SETTINGS_FILE, NULL);
 	fileExists = g_key_file_load_from_file (keyFile, filename, G_KEY_FILE_KEEP_COMMENTS, NULL);
 
-
+	
 	if (fileExists) {
 		GError *error=NULL;
 
-		//BEGIN get list
-		self->priv->c_commands = (gchar **) g_key_file_get_string_list (keyFile, "config", "c_commands", &(self->priv->c_size) ,&error);
-		if (error) {
-			gotAnErrorFlag = TRUE;
-			g_error_free (error);
-			error = NULL;
-		}
-
-		guint consistencyCheck = -1;
-		self->priv->c_titles = g_key_file_get_string_list (keyFile, "config", "c_titles", &consistencyCheck ,&error);
-		if (error) {
-			gotAnErrorFlag = TRUE;
-			g_error_free (error);
-			error = NULL;
-		}
-
-		self->priv->c_current = g_key_file_get_integer (keyFile, "config", "c_current", &error);
-		if (error) {
-			gotAnErrorFlag = TRUE;
-			g_error_free (error);
-			error = NULL;
-		}
-
-		if(consistencyCheck != self->priv->c_size)
-		{
-			g_warning("Settings file corrupted!");
-			gotAnErrorFlag = TRUE;
-		}
-		//END
-
-
-		self->priv->widthRatio = g_key_file_get_double (keyFile, "config", "widthRatio", &error);
-		if (error) {
-// 			gotAnErrorFlag = TRUE;
- 			g_error_free (error);
- 			error = NULL;
-			g_warning("no widthRatio on key file, assuming 1.0");
-			self->priv->widthRatio = 1.0;
-		}
-
-
-		//BEGIN update policy
-		self->priv->updOnClick = g_key_file_get_boolean (keyFile, "config", "updOnClick", &error);
-		if (error) {
-
-			gotAnErrorFlag = TRUE;
-			g_error_free (error);
-			error = NULL;
-		}
-
-		self->priv->updOnDesktop = g_key_file_get_boolean (keyFile, "config", "updOnDesktop", &error);
-		if (error) {
-
-			gotAnErrorFlag = TRUE;
-			g_error_free (error);
-			error = NULL;
-		}
-
-		self->priv->delayIndex = (guint) g_key_file_get_integer (keyFile, "config", "delayIndex", &error);
-		if (error) {
-
-			gotAnErrorFlag = TRUE;
+		gdouble settingsVer = g_key_file_get_double (keyFile, "config", "version", &error);
+		if (error || settingsVer != DESKTOP_CMD_EXEC_SETTINGS_VERSION) {
+			noSettingsFlag = TRUE;
+			noInstanceKeysFlag = TRUE;
 			g_error_free (error);
 			error = NULL;
+			g_warning("Settings file missing or incompatible version");
 		}
 		else
-			self->priv->delaySeconds = GetSeconds(self->priv->delayIndex);
+		{
 
-		self->priv->updNeworkPolicy = (guint) g_key_file_get_integer (keyFile, "config", "updNeworkPolicy", &error);
-		if (error || self->priv->updNeworkPolicy < 0 || self->priv->updNeworkPolicy > 2) {
-			//Not important; in order not to break with older key-file versions, just assume NETWORK__UNRELATED and move on..
-			self->priv->updNeworkPolicy = NETWORK__UNRELATED;
-			
-			g_error_free (error);
-			error = NULL;
+			/*gets "global" data/commands list */
+			//BEGIN get list
+			g_strfreev(self->priv->c_commands);
+			self->priv->c_commands = (gchar **) g_key_file_get_string_list (keyFile, "config", "c_commands", &(self->priv->c_size) ,&error);
+			if (error) {
+				noSettingsFlag = TRUE;
+				g_error_free (error);
+				error = NULL;
+			}
+
+			guint consistencyCheck = -1;
+			g_strfreev(self->priv->c_titles);
+			self->priv->c_titles = g_key_file_get_string_list (keyFile, "config", "c_titles", &consistencyCheck ,&error);
+			if (error) {
+				noSettingsFlag = TRUE;
+				g_error_free (error);
+				error = NULL;
+			}
+
+			if(consistencyCheck != self->priv->c_size)
+			{
+				g_warning("Settings file corrupted!");
+				noSettingsFlag = TRUE;
+			}
+			//END
+
+
+			/*getting instance specific data*/
+			//BEGIN
+			self->priv->widthRatio = g_key_file_get_double (keyFile, self->priv->widgetID, "widthRatio", &error);
+			if (error) {
+				//no value? assume std
+				noInstanceKeysFlag = TRUE;
+				self->priv->widthRatio = 1.0;
+
+				g_warning("no widthRatio on key file, assuming 1.0");
+
+				g_error_free (error);
+				error = NULL;
+			}
+
+			g_free(self->priv->instanceTitle);
+			self->priv->instanceTitle = g_key_file_get_string (keyFile, self->priv->widgetID, "instanceTitle", &error);
+			if (error) {
+				//no value? what now? assume one "fake" for now
+				noInstanceKeysFlag = TRUE;
+				self->priv->instanceTitle = NULL;
+
+				g_warning("no title found");
+
+				g_error_free (error);
+				error = NULL;
+			}
+
+			g_free(self->priv->instanceCmd);
+			self->priv->instanceCmd = g_key_file_get_string (keyFile, self->priv->widgetID, "instanceCmd", &error);
+			if (error) {
+				//no value? what now? assume one "fake" for now
+				noInstanceKeysFlag = TRUE;
+				self->priv->instanceCmd = NULL;
+
+				g_warning("no command found");
+
+				g_error_free (error);
+				error = NULL;
+			}
+
+			//BEGIN update policy
+			self->priv->updOnClick = g_key_file_get_boolean (keyFile, self->priv->widgetID, "updOnClick", &error);
+			if (error) {
+
+				noInstanceKeysFlag = TRUE;
+				g_error_free (error);
+				error = NULL;
+			}
+
+			self->priv->updOnDesktop = g_key_file_get_boolean (keyFile, self->priv->widgetID, "updOnDesktop", &error);
+			if (error) {
+
+				noInstanceKeysFlag = TRUE;
+				g_error_free (error);
+				error = NULL;
+			}
+
+			self->priv->delayIndex = (guint) g_key_file_get_integer (keyFile, self->priv->widgetID, "delayIndex", &error);
+			if (error) {
+
+				noInstanceKeysFlag = TRUE;
+				g_error_free (error);
+				error = NULL;
+			}
+			else
+				self->priv->delaySeconds = GetSeconds(self->priv->delayIndex);
+
+			self->priv->updNeworkPolicy = (guint) g_key_file_get_integer (keyFile, self->priv->widgetID, "updNeworkPolicy", &error);
+			if (error || self->priv->updNeworkPolicy < 0 || self->priv->updNeworkPolicy > 2) {
+				//Not important; in order not to break with older key-file versions, just assume NETWORK__UNRELATED and move on..
+
+				noInstanceKeysFlag = TRUE;
+
+				g_error_free (error);
+				error = NULL;
+			}
+
+			//END
 		}
-
-		
 		
 		//END
 		g_key_file_free (keyFile);
 		g_free (filename);
 	}
 	else
-		gotAnErrorFlag = TRUE;
+		noSettingsFlag = TRUE;
 
-	if(gotAnErrorFlag)
+
+	if(noInstanceKeysFlag)
 	{
-// 		g_warning ("Something happened... resetting all");
 		self->priv->updOnClick = TRUE;
 		self->priv->updOnDesktop = TRUE;
 		self->priv->delayIndex = 0;
@@ -217,28 +274,54 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 
 		self->priv->widthRatio = 1.0;
 
+		self->priv->instanceTitle = NULL;
+		self->priv->instanceCmd = NULL;
+	}
+
+	if(noSettingsFlag)
+	{
 		self->priv->c_commands = NULL;
 		self->priv->c_titles = NULL;
 		self->priv->c_size = 0;
-		self->priv->c_current = 0;
-		desktop_cmd_exec_write_settings(self, TRUE);
 	}
+
+	if(noSettingsFlag || noInstanceKeysFlag)
+	{
+		g_warning ("Problems loading settings. Resetting all");
+		desktop_cmd_exec_write_settings(self, noSettingsFlag, noInstanceKeysFlag);
+	}
+// 	else
+// 		g_warning("Settings file loaded successfully");
 }
 
-void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean reset)
+void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean newAll, gboolean newInstance)
 {
 //  	g_warning ("desktop_cmd_exec_write_settings");
 	GKeyFile *keyFile;
+	gboolean fileExists;
+
 	gchar *fileData;
 	FILE *iniFile;
 	gsize size;
 	gchar *filename;
+
+	if(self->priv->widgetID == NULL)
+	{
+		g_warning("Widget instance not initialized, not writing settings..");
+		return;
+	}
 	
 	keyFile = g_key_file_new();
 
-	if(reset)
+	filename = g_strconcat (HOME_DIR, DESKTOP_CMD_EXEC_SETTINGS_FILE, NULL);
+	fileExists = g_key_file_load_from_file (keyFile, filename, G_KEY_FILE_KEEP_COMMENTS, NULL);
+	
+// 	if (fileExists) {
+// 	}
+
+	if(newAll)
 	{
-// 		g_warning("records empty, filling with pre-loaded vals");
+//  		g_warning("records empty, filling with pre-loaded vals");
 		//records empty, filling with pre-loaded vals
 		self->priv->c_size = 4;
 		gchar * p_titles[] = { "Uptime:", "Battery(%):", "Battery(mAh):", "Boot Reason:", NULL};
@@ -250,34 +333,56 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean reset)
 
 		//clean possible oldies
 		g_strfreev(self->priv->c_titles);
+		self->priv->c_titles = NULL;
 		g_strfreev(self->priv->c_commands);
-
+		self->priv->c_commands = NULL;
 		//assign new from stack values
 		self->priv->c_titles = g_strdupv(p_titles);
 		self->priv->c_commands = g_strdupv(p_commands);
-		
-		self->priv->c_current = 2;
-		self->priv->widthRatio = 1.0;
+	}
 
+	if(newInstance)
+	{
+		g_free(self->priv->instanceTitle);
+		g_free(self->priv->instanceCmd);
+		
+		if((self->priv->c_titles != NULL)
+			&& (self->priv->c_commands != NULL)
+			&& (self->priv->c_size > 0))
+		{
+			self->priv->instanceTitle = g_strdup(self->priv->c_titles[0]);
+			self->priv->instanceCmd = g_strdup(self->priv->c_commands[0]);
+		}
+		
+		self->priv->widthRatio = 1.0;
 		self->priv->updOnClick = TRUE;
 		self->priv->updOnDesktop = TRUE;
 		self->priv->updNeworkPolicy = NETWORK__UNRELATED;
 		self->priv->delayIndex = 0;
-	
 	}
-	
-	g_key_file_set_string_list(keyFile, "config", "c_titles", (const gchar **)(self->priv->c_titles),self->priv->c_size);
-	g_key_file_set_string_list(keyFile, "config", "c_commands", (const gchar **)(self->priv->c_commands),self->priv->c_size);
-	g_key_file_set_integer(keyFile, "config", "c_current", self->priv->c_current);
 
-	g_key_file_set_double (keyFile, "config", "widthRatio", self->priv->widthRatio);
+	g_key_file_set_double (keyFile, "config", "version", DESKTOP_CMD_EXEC_SETTINGS_VERSION);
 
-	g_key_file_set_boolean (keyFile, "config", "updOnClick", self->priv->updOnClick);
-	g_key_file_set_boolean (keyFile, "config", "updOnDesktop", self->priv->updOnDesktop);
-	g_key_file_set_integer (keyFile, "config", "delayIndex", self->priv->delayIndex);
-	g_key_file_set_integer (keyFile, "config", "updNeworkPolicy", self->priv->updNeworkPolicy);
+	if(self->priv->c_titles != NULL && self->priv->c_commands != NULL)
+	{
+		g_key_file_set_string_list(keyFile, "config", "c_titles", (const gchar **)(self->priv->c_titles),self->priv->c_size);
+		g_key_file_set_string_list(keyFile, "config", "c_commands", (const gchar **)(self->priv->c_commands),self->priv->c_size);
+	}
+
+	g_key_file_set_double (keyFile, self->priv->widgetID, "widthRatio", self->priv->widthRatio);
+
+	if(self->priv->instanceTitle != NULL && self->priv->instanceCmd != NULL)
+	{
+		g_key_file_set_string (keyFile, self->priv->widgetID, "instanceTitle", self->priv->instanceTitle);
+		g_key_file_set_string (keyFile, self->priv->widgetID, "instanceCmd", self->priv->instanceCmd);
+	}
+
+	g_key_file_set_boolean (keyFile, self->priv->widgetID, "updOnClick", self->priv->updOnClick);
+	g_key_file_set_boolean (keyFile, self->priv->widgetID, "updOnDesktop", self->priv->updOnDesktop);
+	g_key_file_set_integer (keyFile, self->priv->widgetID, "delayIndex", self->priv->delayIndex);
+	g_key_file_set_integer (keyFile, self->priv->widgetID, "updNeworkPolicy", self->priv->updNeworkPolicy);
 	
-	filename = g_strconcat (HOME_DIR, "/.desktop_cmd_exec", NULL);
+	filename = g_strconcat (HOME_DIR, DESKTOP_CMD_EXEC_SETTINGS_FILE, NULL);
 	fileData = g_key_file_to_data (keyFile, &size, NULL);
 	iniFile = fopen (filename, "w");
 	fputs (fileData, iniFile);
@@ -285,8 +390,20 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean reset)
 	g_key_file_free (keyFile);
 	g_free (fileData);
 	g_free (filename);
-	if(reset)
+
+	if(newAll || newInstance)
+	{
+		g_strfreev(self->priv->c_titles);
+		self->priv->c_titles = NULL;
+		g_strfreev(self->priv->c_commands);
+		self->priv->c_commands = NULL;
+		
+		g_free(self->priv->instanceTitle);
+		self->priv->instanceTitle = NULL;
+		g_free(self->priv->instanceCmd);
+		self->priv->instanceCmd = NULL;
 		desktop_cmd_exec_read_settings(self);
+	}
 }
 
 
@@ -299,6 +416,11 @@ void desktop_cmd_exec_button_press (GtkWidget *widget, GdkEventButton *event, De
 		self->priv->isPressed = TRUE;
 		gtk_widget_queue_draw (GTK_WIDGET (self));
 	}
+
+// 	if(self->priv->widgetID != NULL)
+// 	{
+// 		g_warning("Pressed on %s", self->priv->widgetID);
+// 	}
 }
 
 void desktop_cmd_exec_button_release (GtkWidget *widget, GdkEventButton *event, DesktopCmdExec *self)
@@ -340,8 +462,6 @@ gboolean desktop_cmd_exec_connection_event (ConIcConnection *connection, ConIcCo
 	{
 		self->priv->isConnected = FALSE;
 	}
-
-
 	desktop_cmd_exec_update_content(self);
 	return TRUE;
 }
@@ -387,8 +507,20 @@ void desktop_cmd_exec_content_create (DesktopCmdExec *self)
 
 gboolean desktop_cmd_exec_resize_hack(DesktopCmdExec *self)
 {
-	gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
-	gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+// 	g_warning("HACK");
+	if(self->priv->widgetID != NULL)
+	{
+		desktop_cmd_exec_read_settings (self);
+		desktop_cmd_exec_update_content (self);
+
+		gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+		gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
+	}
+// 	else
+// 	{
+// 		g_warning("HACK unsuccessful");
+// 	}
+
 	return FALSE;
 }
 
@@ -396,28 +528,35 @@ gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self)
 {
 // 	g_warning ("desktop_cmd_exec_update_content");
 
+	if(self->priv->widgetID == NULL)
+	{
+		g_warning("Widget instance unknown... aborting");
+		return;
+	}
+
 	if(self->priv->updNeworkPolicy == NETWORK__ONLY_CONNECTED && self->priv->isConnected == FALSE)
 		return TRUE;
 	
 	if(self->priv->updNeworkPolicy == NETWORK__ONLY_DISCONNECTED && self->priv->isConnected == TRUE)
 		return FALSE;
 
+	
 	FILE *fp;
 	gchar line[256];
 	gchar *result;
 	
 	gboolean found = FALSE;
-	if(self->priv->c_current >= self->priv->c_size)
+	
+	if(self->priv->instanceCmd == NULL || self->priv->instanceTitle == NULL)
 	{
 		gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), "Error:");
 		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), "No commands");
 		return 0;
 	}
-// 	g_warning("To exec %s", self->priv->c_titles[self->priv->c_current]);
 
-	gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), self->priv->c_titles[self->priv->c_current]);
+	gtk_label_set_text (GTK_LABEL (self->priv->cmdTitle_lb), self->priv->instanceTitle);
 
-	fp = popen (self->priv->c_commands[self->priv->c_current], "r");
+	fp = popen (self->priv->instanceCmd, "r");
 	
 	while (fgets (line, sizeof line, fp)) {
 		result = g_strstrip (line);
@@ -461,7 +600,16 @@ static void desktop_cmd_exec_finalize (GObject *object)
 	g_object_unref (self->priv->connection);
 
 	g_strfreev(self->priv->c_titles);
+	self->priv->c_titles = NULL;
+	
 	g_strfreev(self->priv->c_commands);
+	self->priv->c_commands = NULL;
+
+	
+	g_free(self->priv->instanceTitle);
+	self->priv->instanceTitle = NULL;
+	g_free(self->priv->instanceCmd);
+	self->priv->instanceCmd = NULL;
 
 	G_OBJECT_CLASS (desktop_cmd_exec_parent_class)->finalize (object);
 }
@@ -477,11 +625,13 @@ static void desktop_cmd_exec_realize (GtkWidget *widget)
 	gtk_widget_set_app_paintable (widget, TRUE);
 
 	GTK_WIDGET_CLASS (desktop_cmd_exec_parent_class)->realize (widget);
+
+	DesktopCmdExec *self = DESKTOP_CMD_EXEC (widget);
+	self->priv->widgetID = hd_home_plugin_item_get_applet_id (HD_HOME_PLUGIN_ITEM (widget));
 }
 
 static gboolean desktop_cmd_exec_expose_event (GtkWidget *widget, GdkEventExpose *event)
 {
-// 	g_warning ("desktop_cmd_exec_expose_event");
 	DesktopCmdExec *self = DESKTOP_CMD_EXEC (widget);
 
 	cairo_t *cr;
@@ -525,6 +675,7 @@ static gboolean desktop_cmd_exec_expose_event (GtkWidget *widget, GdkEventExpose
 	
 	cairo_destroy(cr);
 
+
 	return GTK_WIDGET_CLASS (desktop_cmd_exec_parent_class)->expose_event (widget, event);
 }
 
@@ -554,6 +705,7 @@ static void desktop_cmd_exec_init (DesktopCmdExec *self)
 	self->priv->updateTimerID = 0;
 	self->priv->isPressed = FALSE;
 
+	self->priv->widgetID = NULL;
 
 	self->priv->connection = con_ic_connection_new ();
 	g_signal_connect (self->priv->connection, "connection-event", G_CALLBACK (desktop_cmd_exec_connection_event), self);
@@ -570,8 +722,6 @@ static void desktop_cmd_exec_init (DesktopCmdExec *self)
 	self->priv->homeWidget = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (self), self->priv->homeWidget);
 	gtk_widget_show (self->priv->homeWidget);
-
-	desktop_cmd_exec_read_settings (self);
 	
 	GdkGeometry hints;
 
@@ -591,9 +741,7 @@ static void desktop_cmd_exec_init (DesktopCmdExec *self)
 // 	gtk_window_resize (GTK_WINDOW (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
 // 	gtk_window_resize (GTK_WINDOW (self), SIZE_WIDTH_ALL, SIZE_HEIGHT_LINE);
 
-	
 	desktop_cmd_exec_content_create (self);
-	desktop_cmd_exec_update_content (self);
 	
 	g_signal_connect (self, "notify::is-on-current-desktop", G_CALLBACK (desktop_cmd_exec_check_desktop), self);
 }
@@ -635,7 +783,8 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 		GtkWidget *cmdSelectionLabel = gtk_label_new ("");
 		gtk_label_set_markup (GTK_LABEL (cmdSelectionLabel), "<small>Command Selection:</small>");
 		gtk_container_add (GTK_CONTAINER (content_area), cmdSelectionLabel);
-		
+
+		desktop_cmd_exec_read_settings(self);
 		
 		//BEGIN Command Selector
 		
@@ -644,10 +793,10 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 		for(i=0;i<self->priv->c_size;i++)
 		{
 			hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), self->priv->c_titles[i]);
+			if(g_strcmp0(self->priv->c_titles[i],self->priv->instanceTitle) == 0)
+				hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, i);
 		}
-		hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, self->priv->c_current);
-
-
+		
 		GtkWidget * cmdSelector = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
 		hildon_button_set_title (HILDON_BUTTON (cmdSelector), "Commands: ");
 		hildon_button_add_title_size_group (HILDON_BUTTON (cmdSelector), group);
@@ -744,10 +893,10 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
 			if(cmdSel < self->priv->c_size && cmdSel >= 0)
 			{
-				self->priv->c_current = (gint) cmdSel;
+				self->priv->instanceTitle = g_strdup(self->priv->c_titles[cmdSel]);
+				self->priv->instanceCmd = g_strdup(self->priv->c_commands[cmdSel]);
 			}
 
-
 			self->priv->widthRatio = gtk_range_get_value(GTK_RANGE(widthSelect));
 
 			gtk_widget_set_size_request (GTK_WIDGET (self), (int)(SIZE_WIDTH_ALL*self->priv->widthRatio), SIZE_HEIGHT_LINE);
@@ -768,22 +917,24 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 			else
 			
 
-			desktop_cmd_exec_write_settings (self, FALSE);
+			desktop_cmd_exec_write_settings (self, FALSE, FALSE);
 			desktop_cmd_exec_update_content (self);
 			settingsRunning = 0;
 		}
 		else if(setDialogReturn == NON_GTK_RESPONSE_ADD_CMD)
 		{
-			desktop_cmd_exec_edit_add_dialog(self, 1);
+			desktop_cmd_exec_edit_add_dialog(self, TRUE, 0);
+			desktop_cmd_exec_write_settings (self, FALSE, FALSE);
 		}
 		else if(setDialogReturn == NON_GTK_RESPONSE_EDIT_CMD)
 		{
 			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
 			if(cmdSel < self->priv->c_size && cmdSel >= 0)
 			{
-				self->priv->c_current = (gint) cmdSel;
+				desktop_cmd_exec_edit_add_dialog(self,FALSE, cmdSel);
+				desktop_cmd_exec_write_settings (self, FALSE, FALSE);
 			}
-			desktop_cmd_exec_edit_add_dialog(self,0);
+
 		}
 		else
 			settingsRunning = 0;
@@ -794,14 +945,14 @@ void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
 	}
 }
 
-void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new)
+void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new, gint curr)
 {
 	GtkWidget *dialog;
 	gchar * title;
 	if(new)
 		title = g_strconcat ("Add new command", NULL);
 	else
-		title = g_strconcat ("Edit ", self->priv->c_titles[self->priv->c_current] ," command", NULL);
+		title = g_strconcat ("Edit ", self->priv->c_titles[curr] ," command", NULL);
 
 	if(new)
 		dialog = gtk_dialog_new_with_buttons (title, NULL, 0, _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, NULL);
@@ -823,7 +974,7 @@ void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new)
 	gtk_box_pack_start (GTK_BOX (titleBox), titleLabel, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (titleBox), titleEntry, TRUE, TRUE, 0);
 	if(!new)
-		hildon_entry_set_text(HILDON_ENTRY (titleEntry), self->priv->c_titles[self->priv->c_current]);
+		hildon_entry_set_text(HILDON_ENTRY (titleEntry), self->priv->c_titles[curr]);
 	
 	gtk_container_add (GTK_CONTAINER (content_area), titleBox);
 	//END
@@ -840,7 +991,7 @@ void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new)
 	gtk_box_pack_start (GTK_BOX (commandBox), commandLabel, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (commandBox), commandEntry, TRUE, TRUE, 0);
 	if(!new)
-		hildon_entry_set_text(HILDON_ENTRY (commandEntry), self->priv->c_commands[self->priv->c_current]);
+		hildon_entry_set_text(HILDON_ENTRY (commandEntry), self->priv->c_commands[curr]);
 	
 	gtk_container_add (GTK_CONTAINER (content_area), commandBox);
 	//END
@@ -856,12 +1007,12 @@ void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new)
 			}
 			else
 			{
-				EditCommand(self, g_strdup(hildon_entry_get_text (HILDON_ENTRY (titleEntry))), g_strdup(hildon_entry_get_text (HILDON_ENTRY (commandEntry))), self->priv->c_current);
+				EditCommand(self, g_strdup(hildon_entry_get_text (HILDON_ENTRY (titleEntry))), g_strdup(hildon_entry_get_text (HILDON_ENTRY (commandEntry))), curr);
 			}
 			break;
 
 		case NON_GTK_RESPONSE_DELETE_CMD:
-			DelCommand(self,self->priv->c_current);
+			DelCommand(self,curr);
 			break;
 
 	}
@@ -887,15 +1038,25 @@ void AddCommand(DesktopCmdExec *self, gchar * s_title, gchar * s_command)
 	}
 	
 	g_strfreev(self->priv->c_titles);
+	self->priv->c_titles = NULL;
 	g_strfreev(self->priv->c_commands);
+	self->priv->c_commands = NULL;
 	
 	newTitles[self->priv->c_size] = s_title;
 	newCommands[self->priv->c_size] = s_command;
 
 	newTitles[self->priv->c_size+1] = NULL;
 	newCommands[self->priv->c_size+1] = NULL;
+
+	g_free(self->priv->instanceTitle);
+	self->priv->instanceTitle = NULL;
+	g_free(self->priv->instanceCmd);
+	self->priv->instanceCmd = NULL;
+	
+	self->priv->instanceTitle = g_strdup(newTitles[self->priv->c_size]);
+	self->priv->instanceCmd = g_strdup(newCommands[self->priv->c_size]);
+
 	
-	self->priv->c_current = self->priv->c_size;
 	self->priv->c_size += 1;
 	
 	self->priv->c_titles = newTitles;
@@ -937,15 +1098,26 @@ void DelCommand(DesktopCmdExec *self, int index)
 
 	//clear old
 	g_strfreev(self->priv->c_titles);
+	self->priv->c_titles = NULL;
 	g_strfreev(self->priv->c_commands);
+	self->priv->c_commands = NULL;
 
 	//assign new
 	self->priv->c_titles = newTitles;
 	self->priv->c_commands = newCommands;
 
-	self->priv->c_current = 0;
-	self->priv->c_size -= 1;//decrement size
+	g_free(self->priv->instanceTitle);
+	self->priv->instanceTitle = NULL;
+	g_free(self->priv->instanceCmd);
+	self->priv->instanceCmd = NULL;
 
+	if(self->priv->c_size > 1)
+	{
+		self->priv->instanceTitle = g_strdup(newTitles[0]);
+		self->priv->instanceCmd = g_strdup(newCommands[0]);
+	}
+	
+	self->priv->c_size -= 1;//decrement size
 }
 
 guint GetSeconds(guint index)
